/* File:  tridiagLUSolve.c
 *        C source code for MEX version of tridiagLUSolve.m
 *
 *  Solve (L*U)*x=b when A = L*U is tridiagonal and diagonally dominant.
 *  The LU factorization is supplied as input.  See tridiagLU
 *  No pivoting is used.  L and U are stored in three 1-D arrays
 *
 *  G.W. Recktenwald,  9 May 2005
 */

#include <math.h>
#include "mex.h"

/* Input Arguments */
#define	E_IN     prhs[0]
#define	F_IN     prhs[1]
#define	B_IN     prhs[2]
#define	D_IN     prhs[3]

/* Output Arguments */
#define	X_OUT    plhs[0]

/* --------  prototypes for functions defined in this file */
void tridiagLUSolve(unsigned int n, double e[], double f[], double b[], double d[], double x[]);

/* -------- Begin mexFunction(), the mex gateway to MATLAB -----------*/

void mexFunction(int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[] )
{
  double	*b,*d,*e,*f,*x;
  long int	n;

  /* -- Check for proper number of arguments */
  if (nrhs != 4) {
    mexErrMsgTxt("tridiagLUSolve requires four input arguments.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("tridiagLUSolve requires one output argument.");
  }

  /* -- Assign pointers to the input parameters */
  e = mxGetPr(E_IN);
  f = mxGetPr(F_IN);
  b = mxGetPr(B_IN);
  d = mxGetPr(D_IN);

  /* -- Create a matrix for the return argument */
	n = mxGetM(D_IN);   /*  Get number of rows in input matrix  */
	X_OUT = mxCreateDoubleMatrix(n, 1, mxREAL);
  x = mxGetPr(X_OUT);
  
  /* -- The tridiagLUSolve function does the work  */
  tridiagLUSolve(n,e,f,b,d,x);
  return;
}

/* ------------------ tridiagLUSolve() --------------------------- */
/*
 *  Solve (L*U)*x=b when A = L*U is tridiagonal and diagonally dominant.
 *  The LU factorization is supplied as input.  See tridiagLU
 *  No pivoting is used.  L and U are stored in three 1-D arrays
 *
 *  Input:  n = (unsigned int) length of input vectors
 *          e[], f[], b[] = (double) vectors holding the LU factors of
 *              the tridiagonal matrix.  e is the subdiaonal of L.  The
 *              main diagonal of L is all ones.  f and b are the main
 *              diagonal and superdiagonal of U
 *          d[] = (double) the right-hand-side vector of the system A*x=b
 *
 *  Return:  x[] = (double) vector containing the solution
 */

void tridiagLUSolve(unsigned int n,
                    double e[],
                    double f[],
                    double b[],
                    double d[],
                    double x[]  )
{
	long int  i;

	/* -- Forward substitution: solve L*y = d.  Use x instead of y  */
	x[0] = d[0];
	for ( i=1; i<n; i++ ) {
		x[i] = d[i] - e[i]*x[i-1];
	}
	/* -- Backward substitution: solve U*x = y.  y is stored in x to start of loop  */
	x[n-1] /= f[n-1];
	for ( i=n-2; i>=0;  i-- ) {
		x[i] = ( x[i] - b[i]*x[i+1] )/f[i];
	}
}