/* File:  tridiagSolve.c
 *        C source code for MEX version of tridiagSolve.m
 *
 *  Solve A*x=b when A is tridiagonal and diagonally dominant.
 *  No pivoting is used.  A is stored as three 1-D arrays
 *
 *  G.W. Recktenwald,  25 April 2005
 */

#include <math.h>
#include "mex.h"

/* Input Arguments */
#define	A_IN     prhs[0]
#define	B_IN     prhs[1]
#define	C_IN     prhs[2]
#define	D_IN     prhs[3]

/* Output Arguments */
#define	X_OUT    plhs[0]

/* --------  prototypes for functions defined in this file */
void tridiagSolve(unsigned int n, double a[], double b[], double c[], double d[], double x[]);

/* -------- Begin mexFunction(), the mex gateway to MATLAB -----------*/

void mexFunction(int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[] )
{
  double	*a,*b,*c,*d,*x;
  long int	n;

  /* -- Check for proper number of arguments */
  if (nrhs != 4) {
    mexErrMsgTxt("tridiagSolve requires four input arguments.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("tridiagSolve requires one output argument.");
  }

  /* -- Assign pointers to the input parameters */
  a = mxGetPr(A_IN);
  b = mxGetPr(B_IN);
  c = mxGetPr(C_IN);
  d = mxGetPr(D_IN);

  /* -- Create a matrix for the return argument */
	n = mxGetM(A_IN);   /*  Get number of rows in input matrix  */
	X_OUT = mxCreateDoubleMatrix(n, 1, mxREAL);
  x = mxGetPr(X_OUT);
  
  /* -- The tridiagSolve function does the work  */
  tridiagSolve(n,a,b,c,d,x);
  return;
}

/* ------------------ tridiagSolve() --------------------------- */
/*
 *  Solve A*x = d where A is a tridiagonal and diagonally dominant
 *  No pivoting is used.  A is stored as three 1-D arrays
 *
 *  Input:  n = (unsigned int) length of input vectors
 *          a[], b[], c[], e[] = (double) vectors defining the
 *              tridiagonal matrix.  a is the main diagonal, b is
 *              the superdiagonal, c is subdiagonal
 *
 *  Return:  x[] = (double) vector containing the solution
 */

void tridiagSolve(unsigned int n,
                  double a[],
                  double b[],
                  double c[],
                  double d[],
                  double x[]  )
{
	long int  i;
	double *f;
	register double e;

	/* -- Allocate temporary variable  */
	f = ( double* ) malloc ( sizeof ( double ) *n );
	if (!f)  mexErrMsgTxt("Failed to allocate temporary f vector in tridiagSolve");

	/* -- Elimination phase: compute e and f */
	/* -- Forward substitution: solve L*y = d.  Use x instead of y  */
	f[0] = a[0];
	x[0] = d[0];
	for ( i=1; i<n; i++ ) {
		e = c[i]/f[i-1];
		f[i] = a[i] - e*b[i-1];
		x[i] = d[i] - e*x[i-1];
	}
	/* -- Backward substitution: solve U*x = y.  y is stored in x to start of loop  */
	x[n-1] /= f[n-1];
	for ( i=n-2; i>=0;  i-- ) {
		x[i] = ( x[i] - b[i]*x[i+1] )/f[i];
	}
	free(f);
}